# AddComment.pl
# Add Comment v1.5 plugin for Movable Type
# http://www.chipple.net/mt/addcomment/
# by Patrick Benny http://www.chipple.net/
# This code cannot be redistributed without permission from the author. 
#
# Version history
#   1.5  - 2007/2/28 - Added optional "Date" field allowing to specify the comment or trackback date/time
#   1.4b - 2006/7/18 - Fixes double declaration that generated warning
#   1.4  - 2006/7/10 - Added "Add A Trackback" form to trackbacks tab
#   1.3  - 2006/7/6  - Added support for Movable Type 3.3
#   1.2  - 2006/7/5  - Silent rebuild, redirect to comments tab
#   1.1  - 2006/6/22 - Added "Set above as [entry's author]" button
#   1.0  - 2006/6/18 - First public release


use strict;
package MT::Plugin::AddComment;

use vars qw( $VERSION );
$VERSION = '1.5';

use MT;
use MT::Plugin;
use MT::Util qw( offset_time_list encode_js );


# Create plugin
my $plugin = MT::Plugin->new({
    name => "Add Comment",
    description => 'Post comments from the comfort of the content management.',
    doc_link => 'http://www.chipple.net/mt/addcomment/',
    author_name => 'Patrick Benny',
    author_link => 'http://www.chipple.net/',
    version => $VERSION,
});
MT->add_plugin($plugin);

## For Movable Type 3.3+
# Use AppTemplateSource to add "Add Comment" form to edit_entry template
MT->add_callback('MT::App::CMS::AppTemplateSource.edit_entry', 9, $plugin, \&_template);
# Use AppTemplateParam to add author variables to edit_entry template
MT->add_callback('MT::App::CMS::AppTemplateParam.edit_entry', 9, $plugin, \&_param);

## For Movable Type 3.2
# Use BigPAPI to add "Add Comment" form to edit_entry template
MT->add_callback('bigpapi::template::edit_entry', 9, $plugin, \&_template);
# Use BigPAPI to add author variables to edit_entry template
MT->add_callback('bigpapi::param::edit_entry', 9, $plugin, \&_param);


#################
### UTILITIES ###
#################



################
### TEMPLATE ###
################

# Add "Add Comment" form to edit_entry template
sub _template {

    my ($cb, $app, $template) = @_;

    my $blog = MT::Blog->load($app->param('blog_id'));
    require POSIX;
    my $server_time_js = POSIX::strftime("%Y/%m/%d %H:%M:%S", offset_time_list(time, $blog));

    my $append;
    $append = '
<script type="text/javascript"><!--
// Calculate time difference with server
var cmnt_iTimeDiff = (new Date()).valueOf()-(new Date("'.$server_time_js.'")).valueOf();
// Set created_on date to current server time
function f_cmnt_SetDateToNow (sFormName,sElemName) {
  var f,el;
  if (  (f = document.forms[sFormName])
     && (el = f.elements[sElemName])) {
    var d = new Date((new Date()).valueOf()-cmnt_iTimeDiff);
    el.value = f_cmnt_FormatMTDate(d);
  }
}
function f_cmnt_Validate (f) {
  var el;
  if (  (el = f.elements["created_on_manual"])
     && el.value != "")
    return f_cmnt_ValidateDate(el.value);
  return true;
}
function f_cmnt_ValidateDate (s) {
// Check if string is well-formated date YYYY-MM?-DD? HH?:II(:SS)?
  var rxDate = new RegExp("^(\\\\d{4}-\\\\d{1,2}-\\\\d{1,2})\\\\s+(\\\\d{1,2}:\\\\d{2}(:\\\\d{2}))?$");
  if (!rxDate.test(s)) {
    alert("'.encode_js($app->translate("Invalid date '[_1]'; authored on dates must be in the format YYYY-MM-DD HH:MM:SS.","%%%")).'".replace("%%%",s));
    return false;
  }
  // Remove all leading zeros for comparison
  var sNoZeros = s.replace(new RegExp("([-:])0([0-9])","g"),"$1$2");
  // Remove extra spaces
  sNoZeros = sNoZeros.replace(rxDate,"$1 $2");
  // Convert input date to date object
  var d = new Date(s.replace(new RegExp("-","g"),"/"));
  // Compare cleaned date string to reformated date object
  if (sNoZeros != d.getFullYear()+"-"+(d.getMonth()+1)+"-"+d.getDate()+" "+d.getHours()+":"+d.getMinutes()+":"+d.getSeconds()) {
    alert("'.encode_js($app->translate("Invalid date '[_1]'; authored on dates should be real dates.","%%%")).'".replace("%%%",s));
    return false;
  }
  return true;
}

// UTIL

// Format MT date
function f_cmnt_FormatMTDate (d) {
  return d.getFullYear()+"-"+f_cmnt_PadTo2Digits(d.getMonth()+1)+"-"+f_cmnt_PadTo2Digits(d.getDate())+" "+f_cmnt_PadTo2Digits(d.getHours())+":"+f_cmnt_PadTo2Digits(d.getMinutes())+":"+f_cmnt_PadTo2Digits(d.getSeconds());
}
// Pad number 
function f_cmnt_PadTo2Digits (i) {
  var s = i.toString();
  return (s.length<2?"0":"")+s;
}

//--></script>
<hr />

<h4><MT_TRANS phrase="Add A Comment"></h4>

<form method="post" action="'.$app->path.$plugin->envelope.'/AddComment.cgi" name="cmnt_form" onsubmit="return f_cmnt_Validate(this);">

<div id="edit-comment">

<input type="hidden" name="id" value="<TMPL_VAR NAME=ID>" />
<input type="hidden" name="blog_id" value="<TMPL_VAR NAME=BLOG_ID>" />
<input type="hidden" name="__mode" value="add_comment" />
<input type="hidden" name="action_name" value="" />
<input type="hidden" name="_type" value="entry" />
<input type="hidden" name="return_args" value="tab=comments&amp;<TMPL_VAR NAME=RETURN_ARGS ESCAPE=HTML>" />
<input type="hidden" name="magic_token" value="<TMPL_VAR NAME=MAGIC_TOKEN>" />

<div class="flt-wrppr">
<div class="leftcol">

<textarea rows="8" cols="30" name="text" id="text" style="height: 12em;"></textarea>

</div>
<div class="rightcol">

<div class="list">
<table cellspacing="0">

<tr class="even">
<td class="label" width="30%"><label for="cmnt_status"><MT_TRANS phrase="Status:"></label></td>
<td width="70%">
<select name="status" id="cmnt_status">
<option value="publish" selected="selected"><MT_TRANS phrase="Published"></option>
<option value="moderate"><MT_TRANS phrase="Unpublished"></option>
</select>
</td>
</tr>

<tr class="odd">
<td class="label"><label for="cmnt_author"><MT_TRANS phrase="Commenter:"></label></td>
<td>
<input name="author" id="cmnt_author" value="" />
</td>
</tr>

<tr class="even">
<td class="label"><label for="cmnt_email"><MT_TRANS phrase="Email:"></label></td>
<td>
<input name="email" id="cmnt_email" value="" />
</td>
</tr>

<tr class="odd">
<td class="label"><label for="cmnt_url"><MT_TRANS phrase="URL:"></label></td>
<td>
<input name="url" id="cmnt_url" value="" />
</td>
</tr>

<tr class="even">
<td class="label" style="vertical-align: top; padding-top: 9px;"><label for="created_on"><MT_TRANS phrase="Date">:</label></td>
<td>
<input name="created_on_manual" id="cmnt_created_on_manual" value="" maxlength="19" style="width: 75%;" />
<small><a href="javascript:f_cmnt_SetDateToNow(\'cmnt_form\',\'created_on_manual\');">Now</a><br />
Leave blank for current time</small>
</td>
</tr>

</table>
</div><!--list-->

</div><!--rightcol-->
</div><!--flt-wrppt-->

</div><!--edit-comment-->

<div class="list-actions">
<div class="button-bar">
<input name="save" type="submit" value="<MT_TRANS phrase="Save Changes">" accesskey="s" title="<MT_TRANS phrase="Save this comment (s)">" />
<input type="hidden" id="me_nickname" value="<TMPL_VAR NAME=USER_NICKNAME ESCAPE=HTML>" />
<input type="hidden" id="me_email" value="<TMPL_VAR NAME=USER_EMAIL ESCAPE=HTML>" />
<input type="hidden" id="me_url" value="<TMPL_VAR NAME=USER_URL ESCAPE=HTML>" />
<input type="button" value="Set commenter as me" onclick="var els=this.form.elements;els[\'author\'].value=document.getElementById(\'me_nickname\').value;els[\'email\'].value=document.getElementById(\'me_email\').value;els[\'url\'].value=document.getElementById(\'me_url\').value;" />
<TMPL_IF NAME=AUTHOR_NICKNAME>
<input type="hidden" id="author_nickname" value="<TMPL_VAR NAME=AUTHOR_NICKNAME ESCAPE=HTML>" />
<input type="hidden" id="author_email" value="<TMPL_VAR NAME=AUTHOR_EMAIL ESCAPE=HTML>" />
<input type="hidden" id="author_url" value="<TMPL_VAR NAME=AUTHOR_URL ESCAPE=HTML>" />
<input type="button" value="Set commenter as <TMPL_VAR NAME=AUTHOR_NICKNAME ESCAPE=HTML>" onclick="var els=this.form.elements;els[\'author\'].value=document.getElementById(\'author_nickname\').value;els[\'email\'].value=document.getElementById(\'author_email\').value;els[\'url\'].value=document.getElementById(\'author_url\').value;" />
</TMPL_IF>
</div>
</div>

</form>
';

    $$template =~ s!(<(div|p) class="message"><MT_TRANS phrase="No comments exist for this entry."></(div|p)>[\r\n]+</TMPL_IF>[\r\n]+)!$1$append!;

    $append = '
<hr />

<h4><MT_TRANS phrase="Add A Trackback"></h4>

<form method="post" action="'.$app->path.$plugin->envelope.'/AddComment.cgi" name="tb_form" onsubmit="return f_cmnt_Validate(this);">

<div id="edit-ping">

<input type="hidden" name="id" value="<TMPL_VAR NAME=ID>" />
<input type="hidden" name="blog_id" value="<TMPL_VAR NAME=BLOG_ID>" />
<input type="hidden" name="__mode" value="add_ping" />
<input type="hidden" name="action_name" value="" />
<input type="hidden" name="_type" value="entry" />
<input type="hidden" name="return_args" value="<TMPL_VAR NAME=RETURN_ARGS ESCAPE=HTML>" />
<input type="hidden" name="magic_token" value="<TMPL_VAR NAME=MAGIC_TOKEN>" />

<div class="flt-wrppr">
<div class="leftcol">

<textarea rows="8" cols="30" name="excerpt" id="excerpt" style="height: 12em;"></textarea>

<div class="list-actions">
<div class="button-bar">
<input name="save" type="submit" value="<MT_TRANS phrase="Save Changes">" accesskey="s" title="<MT_TRANS phrase="Save this TrackBack (s)">" />
<input type="hidden" id="me_blogname" value="<TMPL_VAR NAME=USER_BLOGNAME ESCAPE=HTML>" />
<input type="button" value="Set blog name to self" onclick="var els=this.form.elements;els[\'blog_name\'].value=document.getElementById(\'me_blogname\').value;" />
</div>
</div>

</div>
<div class="rightcol">

<div class="list">
<table cellspacing="0">

<tr class="even">
<td class="label" width="30%"><label for="tb_status"><MT_TRANS phrase="Status:"></label></td>
<td width="70%">
<select name="status" id="tb_status">
<option value="publish" selected="selected"><MT_TRANS phrase="Published"></option>
<option value="moderate"><MT_TRANS phrase="Unpublished"></option>
</select>
</td>
</tr>

<tr class="odd">
<td class="label"><label for="tb_blog_name"><MT_TRANS phrase="Source Site:"></label></td>
<td>
<input name="blog_name" id="tb_blog_name" value="" />
</td>
</tr>

<tr class="even">
<td class="label"><label for="tb_title"><MT_TRANS phrase="Source Title:"></label></td>
<td>
<input name="title" id="tb_title" value="" />
</td>
</tr>

<tr class="odd">
<td class="label"><label for="tb_source_url"><MT_TRANS phrase="URL:"></label></td>
<td>
<input name="source_url" id="tb_source_url" value="" />
</td>
</tr>

<tr class="even">
<td class="label" style="vertical-align: top; padding-top: 9px;"><label for="created_on"><MT_TRANS phrase="Date">:</label></td>
<td>
<input name="created_on_manual" id="tb_created_on_manual" value="" maxlength="19" style="width: 75%;" />
<small><a href="javascript:f_cmnt_SetDateToNow(\'tb_form\',\'created_on_manual\');">Now</a><br />
Leave blank for current time</small>
</td>
</tr>

</table>
</div><!--list-->

</div><!--rightcol-->
</div><!--flt-wrppt-->

</div><!--edit-ping-->

</form>
';

    $$template =~ s!(<(div|p) class="message"><MT_TRANS phrase="No TrackBacks exist for this entry."></(div|p)>[\r\n]+</TMPL_IF>[\r\n]+)!$1$append!;

}

# Add author variables to edit_entry template
sub _param {

    my ($cb, $app, $param) = @_;

    my $author = MT::Author->load($app->{session}->{author_id});
    $param->{'user_nickname'} = $author->nickname;
    $param->{'user_email'} = $author->email;
    $param->{'user_url'} = $author->url;
    $param->{'user_blogname'} = $app->blog->name;

    my $entry = MT::Entry->load($param->{id});
    if ($entry->author_id ne $author->id) {
        use MT::Author;
        my $author = MT::Author->load($entry->author_id);
        $param->{'author_nickname'} = $author->nickname;
        $param->{'author_email'} = $author->email;
        $param->{'author_url'} = $author->url;
    }
}